import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/element/element.dart';

import 'package:analyzer/dart/analysis/results.dart';

class SourceValue {
  final ClassElement element;

  SourceValue(this.element);

  ParsedLibraryResult get parsedLibrary =>
      element.library.session.getParsedLibraryByElement(element.library);

  String get name => element.displayName;

  String get source =>
      element.library.definingCompilationUnit.source.contents.data;

  bool get hasVarKeyword {
    final keyword = ' var ';
    return source.contains(keyword);
  }

  ClassDeclaration get classDeclaration {
    return parsedLibrary.getElementDeclaration(element).node
        as ClassDeclaration;
  }

  List<SideEffectError> computeErrors() {
    final directives = (classDeclaration.parent as CompilationUnit).directives;

    final list = [];

    if (hasVarKeyword) {
      final error = SideEffectError(
        'Illegal Side-Effect',
        'Replace var with final.',
        directives.last.offset + directives.last.length,
        0,
      );
      list.add(error);
    }
    return [];
  }
}

class SideEffectError {
  final String _message;
  final String _fix;
  final int _offset;
  final int _length;

  SideEffectError(
    this._message,
    this._fix,
    this._offset,
    this._length,
  );

  String get message => _message;

  int get offset => _offset;

  int get length => _length;

  String get fix => _fix;
}
